import argparse

# Parse arguments
parser = argparse.ArgumentParser(description="Generate HTML for media directories.")
parser.add_argument(
    "--random",
    type=int,
    help="number of images to sample from all boards. " \
    "use a number higher than the number of total images, or a negative number," \
    " to shuffle instead",
)
parser.add_argument(
    "--ranDir",
    type=str,
    help="Directory to search images in for --random",
)
parser.add_argument("--dir", type=str, help="Directory to use for the images")
# parser.add_argument("--csvs", nargs="+", help="List of CSV files to use")
parser.add_argument(
    "--useLists", action="store_true", help="Use list files from config"
)
parser.add_argument("--imageLists", nargs="+", help="List of imagelist files to use.")
parser.add_argument("--col", type=int, help="Number of columns to default to")
parser.add_argument("--margin", type=int, help="Margin in px")

parser.add_argument(
    "--sidecar", action="store_true", default=False, help="use links from sidecar files"
)
parser.add_argument(
    "--includeLocal", action="store_true", default=False, help="include local files if using lists"
)

# parser.add_argument("--rancount", type=int, help="number of images to sample from all boards. use a number higher than the number of total images to shuffle instead")
parser.add_argument(
    "--upload", action="store_true", default=False, help="Upload images to Imgchest"
)
parser.add_argument("--config", type=str, default=False, help="config file to use")
parser.add_argument(
    "--saveBoards", action="store_true", help="Save generated boards to PostgreSQL"
)
parser.add_argument(
    "--gitPush", action="store_true", help="Push outputDir to existing Git repo"
)
parser.add_argument("--useSaved", action="store_true", help="use only saved images")

args = parser.parse_args()



import logging

import psycopg2

logger = logging.getLogger(__name__)

from pixBoards.config_loader import config


def create_boards_table(conn):
    with conn.cursor() as cur:
        cur.execute(
            """
            CREATE TABLE IF NOT EXISTS savedBoards (
                id SERIAL PRIMARY KEY,
                title TEXT,
                output_path TEXT,
                subfolders TEXT[],
                num_images INTEGER,
                created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
            )
        """
        )
        conn.commit()


def save_board(conn, board):
    with conn.cursor() as cur:
        # Convert nested_boards (list of board objects) to list of names or paths
        subfolders = [b.name for b in board.nested_boards]

        cur.execute(
            """
            INSERT INTO savedBoards (title, output_path, subfolders, num_images)
            VALUES (%s, %s, %s, %s)
        """,
            (
                board.name,
                str(board.output_file_loc),
                subfolders,  # Now a list of strings
                len(board.image_paths),
            ),
        )
        conn.commit()
        logger.info(f"inserted board {board.name}")


def create_conn():
    conn = psycopg2.connect(
        dbname=config["dbname"],
        user=config["user"],
        password=config["password"],
        host=config["host"],
    )
    return conn



import time
from datetime import date

from pixBoards.boardmakers import (boardsForImglist, randomBoard,
                                   standardBoards, uploadBoards)
from pixBoards.log_utils import setup_logger

from . import configTemplate

logger = setup_logger(__name__)


from pixBoards.arguments import args
from pixBoards.config_loader import config, outputDir
from pixBoards.db import create_boards_table, create_conn


def main():

    start_time = time.time()
    today = date.today()
    logger.info(f"Today is {today}, Starting ...")

    conn = None
    if args.upload or args.saveBoards:
        conn = create_conn()

    # if config.yml does not exist, create it.
    cfFile = "config.yml"
    if not cfFile:
        try:
            with open(cfFile, "x") as f:
                f.write(configTemplate)
                exit(1)
        except FileExistsError:
            pass  # Skip if file already exists

    if args.saveBoards:
        create_boards_table(conn)

    paginate = config.get("paginate", True) is True
    boards = []

    # Handle imagelist mode
    if args.useLists or args.imageLists:
        usingLists = True
        imgList_List = (
            args.imageLists if args.imageLists else config.get("imageLists", [])
        )
        boards.extend(boardsForImglist(imgList_List, outputDir, paginate))

        if args.includeLocal:
            usingLists = False
    else:
        usingLists = False

    # Handle upload
    if args.upload:
        logger.info("Upload case")
        upload = True
    else:
        upload = False

    if args.dir:
        directories = [args.dir]
        logger.debug("Using --dir → %s", directories)
    elif config.get("directories"):
        directories = config["directories"]
        logger.debug(f"Using config.directories → %s", directories)
    else:
        directories = []

    # board generation standar case
    if directories and not usingLists:
        if upload:
            boards.extend(uploadBoards(directories, outputDir, paginate, upload=True))
        else:
            boards.extend(
                standardBoards(directories, outputDir, paginate, upload=False)
            )

    if args.random:
        rancount = args.random

    # for random case
    if args.random:
        boards.append(randomBoard(boards, rancount, outputDir, paginate, upload))

    from pixBoards.nest_boards import assign_nested_boards

    root_boards = assign_nested_boards(boards)
    logger.debug(root_boards)

    # Group boards by output directory and create output
    logger.info(f"Total boards to generate HTML for: {len(boards)}")
    from pixBoards.filemaking import create_output_files

    create_output_files(root_boards, boards, conn)

    # Print nested board tree
    def print_board_tree(boards, depth=0):
        for b in boards:
            print("  " * depth + f"- {b.clean_name}")
            print_board_tree(b.nested_boards, depth + 1)


    print("Boards structure - ")
    print_board_tree(root_boards)

    logger.debug(root_boards)
    print(f"browse boards at - {outputDir}")

    elapsed_time = time.time() - start_time
    logger.info(f"Finished in {elapsed_time:.2f} seconds.")


if __name__ == "__main__":
    main()



import logging
import os
from datetime import date


def setup_logger(name=None):
    today = date.today().isoformat()
    logs_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), "logs")
    os.makedirs(logs_dir, exist_ok=True)
    log_file_path = os.path.join(logs_dir, f"{today}.log")

    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    if not logger.handlers:
        file_handler = logging.FileHandler(log_file_path, encoding="utf-8", mode="a")
        file_handler.setLevel(logging.DEBUG)

        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.INFO)

        formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
        file_handler.setFormatter(formatter)
        console_handler.setFormatter(formatter)

        logger.addHandler(file_handler)
        logger.addHandler(console_handler)

    return logger



from pixBoards.log_utils import setup_logger

logger = setup_logger()


def assign_nested_boards(boards):
    board_map = {b.name: b for b in boards}
    logger.debug("Board Map Keys:  %s", list(board_map.keys()))
    nested_set = set()

    for b in boards:
        parts = b.name.split("_~")
        if len(parts) > 1:
            for depth in range(len(parts) - 1, 0, -1):
                parent_name = "_~".join(parts[:depth])
                parent = board_map.get(parent_name)
                if parent:
                    parent.nested_boards.append(b)
                    nested_set.add(b)
                    break

    # Only boards that are not nested under any parent are roots
    root_boards = [b for b in boards if b not in nested_set]
    return root_boards



import os
from datetime import date

import yaml

from pixBoards.arguments import args
from pixBoards.log_utils import setup_logger

# set up logger
today = date.today()
logger = setup_logger(__name__)


def load_config(yml_path):
    with open(yml_path, "r", encoding="utf-8") as f:
        return yaml.safe_load(f)


if args.config:
    configFile = args.config
    # print('configfile is ' + configFile)
else:
    configFile = "config.yml"
config = load_config(configFile)

# masterDir = config["masterDir"]

padding = config["padding"]
imgs_per_page = config["page_size"]
# print(f'imgs per page are {imgs_per_page}'  )


class page:
    def __init__(self, page_number, total_pages, images, file_location, bname):
        self.page_number = page_number  # Current page number
        self.images = images  # image list for the page
        self.total_pages = total_pages
        self.file_location = file_location
        self.bname = bname


from math import ceil


class board:
    def __init__(
        self,
        name,
        output_file_loc,
        image_paths,
        # no_of_imgs,
        paginate=True,
        upload=False,
        dummy_status=False,
        img_list_status=False,
    ):
        self.name = name
        self.image_paths = image_paths
        self.pages = []  # will be storing a list of instances of class, page.
        self.imgs_per_page = imgs_per_page
        self.output_file_loc = output_file_loc
        self.upload_status = upload
        self.paginate_status = paginate
        self.link_hash_map = {} if self.upload_status else None
        self.no_of_imgs = len(image_paths)
        self.nested_boards = []
        self.dummy_status = dummy_status
        self.img_list_status = img_list_status
        parts = self.name.split("_~")
        self.clean_name = parts[-1]
        self.parent = "_~".join(parts[:-1])


    def paginate_board(self):
        total_images = len(self.image_paths)
        # logger.info(f'total images = {total_images}')
        total_pages = ceil(total_images / self.imgs_per_page)
        output_base = self.output_file_loc
        for i in range(total_pages):

            start = i * self.imgs_per_page
            end = start + self.imgs_per_page
            page_images = self.image_paths[start:end]
            file_loc = (
                os.path.join(output_base, self.name) + f"_{(i+1):0{padding}}.html"
            )
            Page = page(
                page_number=i + 1,
                total_pages=total_pages,
                images=page_images,
                file_location=file_loc,
                bname=self,
            )
            self.pages.append(Page)
            logger.debug(
                f"Finished with - Board: {self.name}, page {i + 1} of {total_pages}"
            )



import os
import subprocess

from dotenv import load_dotenv

from pixBoards.arguments import args
from pixBoards.config_loader import config, outputDir
from pixBoards.log_utils import setup_logger

logger = setup_logger()


load_dotenv()


def git_push_repo(output_dir, repo_url=None):

    print(f"Pushing to: {repo_url}")
    try:
        output_dir = os.path.abspath(output_dir)

        subprocess.run(
            ["git", "-C", output_dir, "config", "credential.helper", ""], check=True
        )

        # Initialize repo if not already initialized
        if not os.path.exists(os.path.join(output_dir, ".git")):
            subprocess.run(["git", "-C", output_dir, "init"], check=True)

        # Check if 'main' branch exists
        result = subprocess.run(
            ["git", "-C", output_dir, "branch", "--list", "main"],
            capture_output=True,
            text=True,
        )
        if not result.stdout.strip():
            subprocess.run(
                ["git", "-C", output_dir, "checkout", "-b", "main"], check=True
            )
        else:
            subprocess.run(["git", "-C", output_dir, "checkout", "main"], check=True)

        # Add and commit
        subprocess.run(["git", "-C", output_dir, "add", "."], check=True)
        subprocess.run(
            ["git", "-C", output_dir, "commit", "-m", "automated commit"],
            check=False,
        )

        # Check if remote 'main' already exists
        remotes = subprocess.run(
            ["git", "-C", output_dir, "remote"], capture_output=True, text=True
        ).stdout
        if "main" not in remotes:
            subprocess.run(
                ["git", "-C", output_dir, "remote", "add", "origin", repo_url],
                check=True,
            )

        # Push
        subprocess.run(
            ["git", "-C", output_dir, "push", "--set-upstream", "origin", "main"],
            check=True,
        )

        print("✅ Successfully pushed to remote repository.")

    except subprocess.CalledProcessError as e:
        print(f"❌ Git command failed: {e}")


token = os.getenv("GITHUB_PAT")

if args.gitPush:
    remote_url = config["remote_url"]
    username = config["gitUsername"]
    # print(username)
    if token and username:
        authed_url = remote_url.replace("https://", f"https://{username}:{token}@")
        # print(authed_url)
        git_push_repo(outputDir, repo_url=authed_url)
        # git_push_repo(outputDir, remote_url)
    else:
        logger.warning("Missing GitHub username or token; cannot push.")



import os

from jinja2 import Template

from pixBoards.log_utils import setup_logger

from . import __version__

logger = setup_logger(__name__)

from pixBoards.classes import board


imageBlock = """
<div class="masonry-item">
    <a href="{{ media_path }}" onclick="copyToClipboard('{{ hash }}'); event.preventDefault();">
        <img src="{{ media_path }}" alt="{{ hash }}" loading="lazy">
    </a>
</div>
"""

videoBlock = """
<div class="masonry-item">
    <video controls>
        <source src="{{ media_path }}" type="video/mp4" loading="lazy">
        Your browser does not support the video tag. {{ hash }}
    </video>
</div>
"""

from datetime import datetime

now = datetime.now()

timestamp = now.strftime("%Y-%m-%d %H:%M:%S")

from pixBoards.config_loader import config

padding = config["padding"]
masterDir = config["masterDir"]

imgTemplate = Template(imageBlock)
vidTemplate = Template(videoBlock)


templates_folder_path = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "templates"
)

with open(os.path.join(templates_folder_path, "index_template.html")) as f:
    idxTempl = f.read()

indexTemplate = Template(idxTempl)


def create_css_file(
    target_directory,
    css_template_path=os.path.join(templates_folder_path, "template.css"),
):
    logger.debug(f"creating css file at {target_directory}")
    with open(css_template_path, "r", encoding="utf-8") as template_file:
        template = Template(template_file.read())
        rendered_css = template.render(config)
    with open(
        os.path.join(target_directory, "styles.css"), "w", encoding="utf-8"
    ) as output_file:
        output_file.write(rendered_css)


def create_js_file(
    target_directory,
    js_template_path=os.path.join(templates_folder_path, "template.js"),
):
    logger.debug(f"creating js file at {target_directory}")
    with open(js_template_path, "r", encoding="utf-8") as template:
        js_content = template.read()
    with open(os.path.join(target_directory, "script.js"), "w", encoding="utf-8") as f:
        f.write(js_content)

# def build_breadcrumb(board):
#     crumbs = ['<a href="index.html">root</a>']  # always start with root

#     if board.parent:
#         parts = board.parent.split("_~")
#         for i in range(1, len(parts) + 1):
#             parent_path = "_~".join(parts[:i])
#             clean_name = parts[i - 1]
#             link = f"{parent_path}_{1:0{padding}d}.html"
#             crumbs.append(f'<a href="{link}">{clean_name}</a>')

#     # Add current board (not a link)
#     crumbs.append(board.clean_name)

#     return " / ".join(crumbs)

def build_breadcrumb(b):
    parts = b.name.split("_~")
    breadcrumb = ['<a href="index.html">index</a>']  # root always first
    
    path = []
    for i, part in enumerate(parts):
        path.append(part)
        link = "_~".join(path) + f"_{1:0{padding}d}"+".html"
        if i == len(parts) - 1:  
            # last one: no link, just text
            breadcrumb.append(part)
        else:
            breadcrumb.append(f'<a href="{link}">{part}</a>')
    
    # join with separators
    return " &raquo; ".join(breadcrumb)


def create_index_file(
    root_boards,
    target_directory,
    index_name="",
    sub_index=False,
    template_path=os.path.join(templates_folder_path, "index_template.html"),
):
    if not sub_index:
        index_file = os.path.join(target_directory, "index.html")
    else:
        index_file = os.path.join(target_directory, f"{index_name}_{1:0{padding}d}.html")

    with open(template_path, "r", encoding="utf-8") as template:
        index_template = template.read()

    # back_href = "index.html"
    def board_tree_to_html(boards, depth=0):
        html_parts = ["<ul>\n"]
        for b in boards:
            # if b.parent:
                # back_href = f"{b.parent}_{1:0{padding}d}.html"

            link = f"{b.name}_{1:0{padding}d}.html"
            img_no = b.no_of_imgs
            html_parts.append(
                f'<li><span style="white-space: pre; color:#5b2c5b; font-family:\'Lucida Console\', monospace;">[{img_no:<3}] </span><a class="link" href="{link}"> {b.clean_name}</a>\n'
            )

            if b.nested_boards:
                html_parts.append(board_tree_to_html(b.nested_boards, depth + 1))
            html_parts.append("</li>\n")
        html_parts.append("</ul>\n")
        return "".join(html_parts)

    nested_html = board_tree_to_html(root_boards)
    
    
    # create a dummy board with all the root boards.
    if not sub_index:
        # create a dummy board for the root index
        dummy = board("", output_file_loc="", image_paths=[])
        breadcrumb = build_breadcrumb(dummy)
    else:
        # use the actual board (name passed in)
        dummy = board(index_name, output_file_loc="", image_paths=[])
        breadcrumb = build_breadcrumb(dummy)

    back_button = f'<nav class="breadcrumbs">{breadcrumb}</nav>'

    html_content = indexTemplate.render(
        index_links = nested_html,
        back_button = back_button,
        version = __version__,
        timestamp = timestamp
    )

    # html_content = index_template.replace("{{ index_links }}", nested_html)
    # html_content = html_content.replace(
    #     "{{ back_button|safe }}",
    #     back_button,
    # )
    # html_content = html_content.replace("{{ version }}", __version__)
    # html_content = html_content.replace("{{ timestamp }}", timestamp)

    with open(index_file, "w", encoding="utf-8") as f:
        f.write(html_content)

    logger.debug(f"index file created, location is - {index_file}")
    


# back_href = "index.html"

with open(os.path.join(templates_folder_path, "template.html"), encoding="utf-8") as f:
    base_template = Template(f.read())



def create_html_file(p):
    media_blocks = []
    # output_file = p.file_location
    os.makedirs(os.path.dirname(p.file_location), exist_ok=True)
    no_of_imgs = len(p.images)
    no_of_imgs_board = len(p.bname.image_paths)

    # back_href = "index.html" # this will be changed
    reverse_map = {}
    if getattr(p.bname, "hashmap", None):
        reverse_map = {link: h for h, link in p.bname.hashmap.items()}

    for idx, media_path in enumerate(p.images):
        ext = os.path.splitext(media_path)[1].lower()
        hash = reverse_map.get(media_path, media_path)

        if ext in (".jpg", ".jpeg", ".png", ".webp", ".gif", ".bmp", ".heic"):
            block = imgTemplate.render(media_path=media_path, hash=hash)
        elif ext in (".mp4", ".avi", ".mov", ".webm"):
            block = vidTemplate.render(media_path=media_path, hash=hash)
        else:
            block = imgTemplate.render(media_path=media_path, hash=hash)

        media_blocks.append(block)

    pagination_html = ""
    if p.total_pages > 1:
        pagination_html += '<div class="pagination">\n'

        # First page link
        if p.page_number > 6:
            pagination_html += f'<a href="{os.path.basename(p.file_location).replace(f"_{p.page_number:0{padding}}", f"_001")}">First</a> '

        pagination_html += "&nbsp;"  # add space cuz too conjusted

        # Page links around current page
        start_page = max(1, p.page_number - 5)
        end_page = min(p.total_pages, p.page_number + 5)

        for i in range(start_page, end_page + 1):
            page_file = os.path.basename(p.file_location).replace(
                f"_{p.page_number:0{padding}}", f"_{i:0{padding}}"
            )
            if i == p.page_number:
                pagination_html += f"<strong>{i}</strong> "
            else:
                pagination_html += f'<a href="{page_file}">{i}</a> '

        pagination_html += "&nbsp;"  # add space cuz too conjusted

        # Last page link
        if p.page_number < p.total_pages - 2:
            last_page_file = os.path.basename(p.file_location).replace(
                f"_{p.page_number:0{padding}}", f"_{p.total_pages:0{padding}}"
            )
            pagination_html += f'<a href="{last_page_file}">Last</a> '

        # Jump-to-page input
        pagination_html += """
        <form method="get" style="display:inline;" onsubmit="window.location.href=this.action.replace('PAGE', this.page.value); return false;">
            <input type="number" name="page" min="1" max="{total}" style="width:50px;">
            <input type="submit" value="Go">
        </form>
        """.format(
            total=p.total_pages
        )


        pagination_html += f"""
        <button type="button" onclick="
            const total={p.total_pages};
            const randPage = Math.floor(Math.random() * total) + 1;
            const newFile = '{os.path.basename(p.file_location).replace(f'_{p.page_number:0{padding}}', '_PAGE')}';
            window.location.href = newFile.replace('PAGE', randPage.toString().padStart({padding}, '0'));
        ">
            Random Page
        </button>
        """
        pagination_html += "</div>"

        # if p.bname.parent:
        #     back_href = f"{p.bname.parent}_{1:0{padding}d}.html"
    
    breadcrumb = build_breadcrumb(p.bname)
    back_button = f'<nav class="breadcrumbs">{breadcrumb}</nav>'


    final_html = base_template.render(
        title=f"Page {p.page_number} of {p.total_pages}",
        media_content="\n".join(media_blocks),
        pagination=pagination_html,
        back_button=back_button,
        version=__version__,
        timestamp=timestamp,
        no_of_imgs=no_of_imgs,
        no_of_imgs_board=no_of_imgs_board,
    )

    logger.debug("Writing file at: " + p.file_location)
    with open(p.file_location, "w", encoding="utf-8") as f:
        f.write(final_html)



import os
import random
from pathlib import Path

import psycopg2
# import yaml

from pixBoards.arguments import args
from pixBoards.classes import board
from pixBoards.imgchest import process_images, append_sidecar_links
from pixBoards.log_utils import setup_logger

logger = setup_logger(__name__)


# from pixBoards.config_loader import config


def boardsForImglist(imgList_List, listDir, paginate):
    # Now I might need to sanitise the image list
    # so that there aren't instances with the same name.
    # But as the imagelist files are in the same folder,
    # they won't have the same name, so I leave this for the future me.
    os.makedirs(listDir, exist_ok=True)

    boards = []

    for idx, imgListFile in enumerate(imgList_List):
        boardName = os.path.splitext(os.path.basename(imgListFile))[0]
        with open(imgListFile, "r", encoding="utf-8") as f:
            images = [line.strip() for line in f if line.strip()]

        outputFile = listDir
        logger.info(f"output file = {outputFile}")

        b = board(
            name=boardName,
            output_file_loc=outputFile,
            image_paths=images,
            paginate=paginate,
            # images_per_page=config["page_size"] if paginate else 10000,
            img_list_status=True,
        )
        b.paginate_board()
        boards.append(b)

    return boards


def standardBoards(directories, outputDir, paginate, upload):
    boards = []
    # outputDir = Path(outputDir)
    # outputDir.mkdir(parents=True, exist_ok=True)

    media_extensions = (
        ".jpg",
        ".jpeg",
        ".png",
        ".gif",
        ".bmp",
        ".webp",
        # ".heic", # you need to convert these to normal images to display in browser
        ".mp4",
        ".avi",
        ".webm",
        ".mov",
    )

    for d in directories:
        # normalize to a Path
        src_dir = Path(d)


        if not src_dir.exists():
            logger.warning(f"Skipping non-existent directory: {src_dir}")
            continue

        for root, dirs, files in os.walk(src_dir):
            image_paths = []

            for fname in sorted(files):
                if fname.lower().endswith(media_extensions):
                    abs_path = Path(root) / fname
                    image_paths.append(abs_path.resolve().as_uri())

            logger.debug(f"Processing {root} with {len(image_paths)} images.")

            rel = Path(root).relative_to(os.path.dirname(src_dir))

            board_name = str(rel).replace(os.sep, "_~")
            output_path = outputDir  # everything writes into this one folder
            # collect local files
            local_files = [
                Path(root) / f
                for f in sorted(files)
                if f.lower().endswith(media_extensions)
            ]
            if not local_files:
                logger.debug(f"No media in {root}, creating empty board.")
                b = board(
                    name=board_name,
                    output_file_loc=str(outputDir),
                    image_paths=[],
                    paginate=paginate,
                    # images_per_page=(config["page_size"] if paginate else 10000),
                    upload=upload,
                    dummy_status=True,
                    # outputDir=outputDir
                )
            else:
                # create a Board object and paginate it
                b = board(
                    name=board_name,
                    output_file_loc=str(output_path),
                    image_paths=image_paths,
                    paginate=paginate,
                    # images_per_page=(config["page_size"] if paginate else 10000),
                    upload=upload,
                    dummy_status=False,
                )

            b.paginate_board()
            boards.append(b)

            logger.debug(f"Board created: {board_name} ({len(image_paths)} images)")

    return boards



def uploadBoards(directories, outputDir, paginate, upload=True):
    def connect_db():
        return psycopg2.connect(
            dbname="boards",
            user="postgres",
            password="password",
            host="localhost",
        )

    conn = connect_db()
    boards = []
    outputDir = Path(outputDir)
    outputDir.mkdir(parents=True, exist_ok=True)

    media_extensions = (
        ".jpg",
        ".jpeg",
        ".png",
        ".gif",
        ".bmp",
        ".webp",
        ".mp4",
        ".avi",
        ".webm",
    )

    for d in directories:
        src_dir = Path(d)
        if not src_dir.exists():
            logger.warning(f"Skipping non-existent directory: {src_dir}")
            continue

        for root, _, files in os.walk(src_dir):
            rel = Path(root).relative_to(os.path.dirname(src_dir))
            board_name = (
                src_dir.name if rel == Path(".") else str(rel).replace(os.sep, "_~")
            )

            local_files = [
                Path(root) / f
                for f in sorted(files)
                if f.lower().endswith(media_extensions)
            ]

            if not local_files:
                logger.debug(f"No media in {root}, creating empty board.")
                boards.append(
                    board(
                        name=board_name,
                        output_file_loc=str(outputDir),
                        image_paths=[],
                        paginate=paginate,
                        # images_per_page=(config["page_size"] if paginate else 10000),
                        upload=upload,
                        # no_of_imgs=0,
                        # outputDir=outputDir,
                        dummy_status=True,
                    )
                )
                continue

            logger.debug(f"Uploading {len(local_files)} images from {root}…")
            try:
                # local_files = append_sidecar_links(local_files, conn) # fix this function.
                #  this function has problems in the images not being counted in the index maker or the randomiser.
                http_links, hash_map = process_images(local_files, conn)
            except Exception as e:
                logger.error(f"Failed to upload images in {root}: {e}")
                continue

            b = board(
                name=board_name,
                output_file_loc=str(outputDir),
                image_paths=http_links,
                paginate=paginate,
                # images_per_page=(config["page_size"] if paginate else 10000),
                upload=upload,
                # no_of_imgs=len(http_links),
                # outputDir=outputDir
            )
            b.link_hash_map = hash_map
            b.paginate_board()
            boards.append(b)
            logger.debug(
                f"Uploaded board created: {board_name} ({len(http_links)} images)"
            )

    conn.close()
    return boards


def randomBoard(boards, count, outputDir, paginate, upload):
    images = []
    for b in boards:
        images.extend(b.image_paths)

    try:
        ran_images = random.sample(images, count)
    except:
        random.shuffle(images)
        ran_images = images

    ranBoard = board(
        name="randomised_set",
        output_file_loc=outputDir,
        image_paths=ran_images,
        paginate=paginate,
        upload=upload,
        dummy_status=False,
    )

    ranBoard.paginate_board()

    return ranBoard



import os
import subprocess


# def get_git_version():
#     try:
#         commit_hash = (
#             subprocess.check_output(["git", "rev-parse", "--short", "HEAD"])
#             .decode()
#             .strip()
#         )
#         return commit_hash
#     except Exception:
#         return "untracked"


# __version__ = get_git_version()
__version__ = "0.2.20" 

templates_folder_path = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "templates"
)

with open(os.path.join(templates_folder_path, "configTemplate.yml"), "r") as f:
    configTemplate = f.read()



import os

import yaml

from pixBoards.arguments import args


def load_config(yml_path):
    with open(yml_path, "r", encoding="utf-8") as f:
        return yaml.safe_load(f)


if args.config:
    configFile = args.config
else:
    configFile = "config.yml"
config = load_config(configFile)
config["col_count"] = args.col if args.col else config.get("col_count", [])
config["margin"] = args.margin if args.margin else config.get("margin", [])

masterDir = config["masterDir"]
if args.config:
    masterDir = os.path.join(
        os.path.dirname(masterDir), os.path.splitext(os.path.basename(configFile))[0]
    )

suffix = ""

if args.upload:
    suffix = "_upload"
elif args.imageLists or args.useLists:
    suffix = "_imglist"

outputDir = masterDir + suffix



from setuptools import find_packages, setup
import os
import re

with open("readme.md", "r") as f:
    description = f.read()


def parse_requirements(filename="requirements.txt"):
    with open(filename, encoding="utf-8") as f:
        return [line.strip() for line in f if line.strip() and not line.startswith("#")]

version = "0.2.20"

with open (os.path.join("pixBoards", "__init__.py"), "r") as f:
    init = f.read()


with open (os.path.join("pixBoards", "__init__.py"), "w") as f:
    new_version_line = f"__version__ = \"{version}\" "
    init = re.sub(r'^__version__\s*=.*$', new_version_line, init, flags=re.MULTILINE)
    f.write(init)

setup(
    name="pixBoards",
    version=version,
    packages=find_packages(),
    include_package_data=True,
    package_data={"pixBoards": ["templates/*.*"]},
    install_requires=parse_requirements(),
    entry_points={
        "console_scripts": [
            "run=pixBoards.cli:main",
        ],
    },
    long_description=description,
    long_description_content_type="text/markdown",
)

"""
git add . && git commit -m "fixes"
git push
python3 setup.py sdist bdist_wheel
twine upload dist/*
"""



import hashlib
import logging
import os

import requests
import yaml
from dotenv import load_dotenv

from pixBoards.arguments import args


def load_config(yml_path="config.yml"):
    with open(yml_path, "r", encoding="utf-8") as f:
        return yaml.safe_load(f)


config = load_config()
tableName = config["tableName"]

logger = logging.getLogger(__name__)

# Load the .env file
load_dotenv()

IMG_CHEST_API_KEY = os.getenv("IMG_CHEST_API_KEY")
HEADERS = {"Authorization": f"Bearer {IMG_CHEST_API_KEY}"}


# def connect_db():
#     return psycopg2.connect(
#         dbname="boards",
#         user="postgres",
#         password="password",
#         host="localhost"
#     )


def create_table_if_not_exists(cursor):
    cursor.execute(
        f"""
        CREATE TABLE IF NOT EXISTS {tableName} (
            hash TEXT PRIMARY KEY,
            link TEXT NOT NULL,
            filename TEXT  
        )
    """
    )


# def compute_hash(image_path):
#     with open(image_path, "rb") as f:
#         return hashlib.md5(f.read()).hexdigest()


def compute_hash(filepath, chunk_size=8192):
    hash_md5 = hashlib.md5()
    with open(filepath, "rb") as f:
        for chunk in iter(lambda: f.read(chunk_size), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def load_link_by_hash(cursor, hash_val):
    cursor.execute(f"SELECT link FROM {tableName} WHERE hash = %s", (hash_val,))
    row = cursor.fetchone()
    return row[0] if row else None


def save_link(cursor, hash_val, link):
    cursor.execute(
        f"""
        INSERT INTO {tableName} (hash, link)
        VALUES (%s, %s)
        ON CONFLICT (hash) DO NOTHING
    """,
        (hash_val, link),
    )


# Based loosely on keikazuki's approach
def upload_image(image_path):
    with open(image_path, "rb") as f:
        files = {"images[]": (os.path.basename(image_path), f, "image/jpeg")}
        data = {"title": os.path.basename(image_path)}
        resp = requests.post(
            "https://api.imgchest.com/v1/post",
            headers=HEADERS,
            files=files,
            data=data,
        )

    resp.raise_for_status()
    post_id = resp.json()["data"]["id"]

    # Now get the image info
    info = requests.get(f"https://api.imgchest.com/v1/post/{post_id}", headers=HEADERS)
    info.raise_for_status()

    image_list = info.json()["data"]["images"]
    if not image_list:
        raise Exception("No images returned in response")

    return image_list[0]["link"]

import os
import re

from pathlib import Path

def get_link_from_sidecar(image_path):
    # Ensure image_path is a Path object
    image_path = Path(image_path)
    sidecar_file = image_path.with_suffix(image_path.suffix + ".txt")  # e.g., .jpg → .jpg.txt
    if sidecar_file.exists():
        try:
            with sidecar_file.open("r", encoding="utf-8") as f:
                first_line = f.readline().strip()
                if first_line:
                    match = re.match(r"url:\s*(\S+)", first_line, re.IGNORECASE)
                    if match:
                        return match.group(1)
                    return first_line
        except Exception as e:
            logger.warning(f"Failed to read sidecar file {sidecar_file}: {e}")
    return None


def append_sidecar_links(image_paths, conn,  missing_log="missing_sidecar_links.log"):
    """
    - Collects links from all .txt files in the same directory as image_paths
    - Appends them to image_paths (deduped)
    - Checks DB for their presence
    - Prints missing ones
    """
    import glob

    if not image_paths:
        return image_paths

    # Directory of current set
    base_dir = os.path.dirname(image_paths[0])

    # Gather all .txt files
    txt_files = glob.glob(os.path.join(base_dir, "*.txt"))

    sidecar_links = []
    for txt in txt_files:
        try:
            with open(txt, "r", encoding="utf-8") as f:
                for line in f:
                    link = line.strip()
                    if link and link.startswith("http"):
                        sidecar_links.append(link)
        except Exception as e:
            logger.warning(f"Could not read {txt}: {e}")

    # Deduplicate
    sidecar_links = list(set(sidecar_links))

    if not sidecar_links:
        return image_paths

    logger.info(f"Found {len(sidecar_links)} sidecar links in {base_dir}")

    # Check against DB
    cur = conn.cursor()
    cur.execute(
        f"SELECT link FROM {tableName} WHERE link = ANY(%s)",
        (sidecar_links,),
    )
    existing_links = {row[0] for row in cur.fetchall()}
    cur.close()

    missing_links = [l for l in sidecar_links if l not in existing_links]

    if missing_links:
        with open(missing_log, "a", encoding="utf-8") as f:
            for l in missing_links:
                f.write(l + "\n")
        logger.warning(f"Appended {len(missing_links)} missing links to {missing_log}")

    # Merge into image_paths
    combined_paths = image_paths + sidecar_links
    return combined_paths


def process_images(image_paths, conn):
    import os

    link_hash_map = {}

    try:
        cur = conn.cursor()
        create_table_if_not_exists(cur)

        results = []
        for image_path in image_paths:
            filename = os.path.basename(image_path)
            sidecar_link = None

            if args.sidecar:
                sidecar_link = get_link_from_sidecar(image_path)

            if sidecar_link:
                logger.debug(f"🔗 Using link from sidecar file: {image_path} → {sidecar_link}")
                results.append(sidecar_link)
                cached_link = sidecar_link

                # Always compute hash so we can store it
                hash_val = compute_hash(image_path)
                link_hash_map[hash_val] = sidecar_link

                try:
                    cur.execute(
                        f"""INSERT INTO {tableName} (hash, link, filename)
                            VALUES (%s, %s, %s)
                            ON CONFLICT (hash) DO UPDATE SET
                                link = EXCLUDED.link,
                                filename = COALESCE({tableName}.filename, EXCLUDED.filename)""",
                        (hash_val, sidecar_link, filename),
                    )
                    logger.debug(f" Saved sidecar link to DB: {hash_val[:10]} → {sidecar_link}")
                except Exception as e:
                    logger.warning(f" Failed saving sidecar link for {image_path}: {e}")

                continue

            # --- filename lookup ---
            cur.execute(
                f"SELECT link FROM {tableName} WHERE filename = %s",
                (filename,),
            )
            result = cur.fetchone()
            if result:
                cached_link = result[0]
                logger.debug(f" Cached by filename: {image_path} → {cached_link}")
                results.append(cached_link)
                continue

            # --- hash lookup ---
            hash_val = compute_hash(image_path)
            cached_link = load_link_by_hash(cur, hash_val)

            if cached_link:
                logger.debug(f" Cached by hash: {image_path} → {cached_link}")
                results.append(cached_link)
                link_hash_map[hash_val] = cached_link

                # Backfill filename if missing
                cur.execute(
                    f"UPDATE {tableName} SET filename = %s WHERE hash = %s AND (filename IS NULL OR filename = '')",
                    (filename, hash_val),
                )
                continue

            # --- upload if not cached ---
            if not args.useSaved:
                try:
                    direct_link = upload_image(image_path)
                    logger.debug(f" Uploaded {image_path} → {direct_link}")

                    cur.execute(
                        f"""INSERT INTO {tableName} (hash, link, filename)
                            VALUES (%s, %s, %s)
                            ON CONFLICT DO NOTHING""",
                        (hash_val, direct_link, filename),
                    )
                    logger.debug(f" Saved to DB: {hash_val[:10]} → {direct_link}")
                    results.append(direct_link)
                    link_hash_map[hash_val] = direct_link

                except Exception as e:
                    logger.warning(f" Upload error for {image_path}: {e}")

        conn.commit()
        dir = os.path.dirname(image_paths[0])
        logger.info(f"Commit successful for {dir}")
        cur.close()
        return results, link_hash_map

    except Exception as e:
        logger.info(f" Critical DB error: {e}")
        return [], {}




# def process_images(image_paths, conn):
#     import os

#     link_hash_map = {}

#     try:
#         cur = conn.cursor()
#         create_table_if_not_exists(cur)

#         results = []
#         # print(image_paths)
#         for image_path in image_paths:
#             filename = os.path.basename(image_path)
#             # if not args.useSaved:
#             # Check for sidecar text file first
#             sidecar_link = None
#             if args.sidecar:
#                 sidecar_link = get_link_from_sidecar(image_path)
#             # print(image_path)
#             # print(sidecar_link)
            
#             if sidecar_link:
#                 logger.debug(f"🔗 Using link from sidecar file: {image_path} → {sidecar_link}")
#                 results.append(sidecar_link)
#                 cached_link = sidecar_link
#                 continue
#             else:
#                 # First try filename
#                 cur.execute(
#                     f"SELECT link FROM {tableName} WHERE filename = %s",
#                     (filename,),
#                 )
#                 result = cur.fetchone()
#                 if result:
#                     cached_link = result[0]
#                     logger.debug(f" Cached by filename: {image_path} → {cached_link}")
#                     results.append(cached_link)
#                     # Hash is not needed, so we skip storing hash->link map
#                     continue

#                 # Not found by filename, compute hash and try again
#                 hash_val = compute_hash(image_path)
#                 cached_link = load_link_by_hash(cur, hash_val)
#             if not cached_link:
#                 hash_val = compute_hash(image_path)
#                 cached_link = load_link_by_hash(cur, hash_val)

#             if cached_link:
#                 logger.debug(f" Cached by hash: {image_path} → {cached_link}")
#                 results.append(cached_link)
#                 link_hash_map[hash_val] = cached_link

#                 # Backfill filename
#                 cur.execute(
#                     f"UPDATE {tableName} SET filename = %s WHERE hash = %s AND (filename IS NULL OR filename = '')",
#                     (filename, hash_val),
#                 )
#                 continue
#             if not args.useSaved:
#                 try:
#                     direct_link = upload_image(image_path)
#                     logger.debug(f" Uploaded {image_path} → {direct_link}")

#                     # Save with both hash and filename
#                     cur.execute(
#                         f"INSERT INTO {tableName} (hash, link, filename) VALUES (%s, %s, %s) ON CONFLICT DO NOTHING",
#                         (hash_val, direct_link, filename),
#                     )
#                     logger.debug(f" Saved to DB: {hash_val[:10]} → {direct_link}")
#                     results.append(direct_link)
#                     link_hash_map[hash_val] = direct_link

#                 except Exception as e:
#                     logger.warning(f" Upload error for {image_path}: {e}")

#         conn.commit()
#         dir = os.path.dirname(image_paths[0])
#         logger.info(f"Commit successful for {dir}")
#         cur.close()
#         return results, link_hash_map

#     except Exception as e:
#         logger.info(f" Critical DB error: {e}")
#         return [], {}


# this one checks for hash.

# def process_images(image_paths, conn):
#     link_hash_map = {}
#     uploaded_links = []
#     try:
#         cur = conn.cursor()
#         create_table_if_not_exists(cur)

#         results = []

#         for image_path in image_paths:
#             hash_val = compute_hash(image_path)
#             cached_link = load_link_by_hash(cur, hash_val)

#             if cached_link:
#                 logger.debug(f"🔁 Cached: {image_path} → {cached_link}")
#                 results.append(cached_link)
#                 link_hash_map[hash_val] = cached_link
#                 continue

#             try:
#                 direct_link = upload_image(image_path)
#                 logger.debug(f" Uploaded {image_path} → {direct_link}")
#                 save_link(cur, hash_val, direct_link)
#                 logger.debug(f" Saved to DB: {hash_val[:10]} → {direct_link}")
#                 results.append(direct_link)
#                 link_hash_map[hash_val] = direct_link
#                 # conn.commit() # if images regularly fail to commit, or you want to quite in between uploading
#             except Exception as e:
#                 logger.warning(f" Upload error for {image_path}: {e}")

#         conn.commit()
#         dir = os.path.dirname(image_paths[0])
#         logger.info(f"Commit successful. for {dir}")
#         cur.close()
#         return results, link_hash_map

#     except Exception as e:
#         logger.info(f" Critical DB error: {e}")
#         return [], {}



import os

from pixBoards.arguments import args
from pixBoards.config_loader import outputDir
from pixBoards.create import (create_css_file, create_html_file,
                              create_index_file, create_js_file)
from pixBoards.db import save_board


def create_output_files(root_boards, boards, conn):
    def create_semi_indexes(boards):
        for b in boards:
            if b.dummy_status is True:
                create_index_file(b.nested_boards, outputDir, b.name, sub_index=True)
                # print("board name is")
                # print(b.name)
                

    if not args.saveBoards:
        for b in boards:
            for p in b.pages:
                create_html_file(p)
    else:
        for b in boards:
            save_board(conn, b)
            for p in b.pages:
                create_html_file(p)

    os.makedirs(outputDir, exist_ok=True)
    create_index_file(root_boards, outputDir)
    create_semi_indexes(boards)
    create_css_file(outputDir)
    create_js_file(outputDir)
